From 6d35e7a4238a8f77d438f7dce7dac9b631b3cef1 Mon Sep 17 00:00:00 2001
From: Marvin Scholz <epirat07@gmail.com>
Date: Fri, 4 Jan 2019 00:13:04 +0100
Subject: [PATCH] compat: work-around busted FreeBSD qsort_r()
MIME-Version: 1.0
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: 8bit

Co-authored-by: Rémi Denis-Courmont <remi@remlab.net>
---
 configure.ac         | 18 ++++++++++++++++++
 include/vlc_fixups.h | 14 ++++++++++++++
 2 files changed, 32 insertions(+)

diff --git a/configure.ac b/configure.ac
index 1b63a74736..4b68219a8c 100644
--- a/configure.ac
+++ b/configure.ac
@@ -747,6 +747,24 @@ AC_CHECK_LIB([m], [sincos], [
   AC_LIBOBJ([sincos])
 ])
 
+AC_CHECK_FUNC([qsort_r], [
+    AC_MSG_CHECKING([broken qsort_r prototype])
+    AC_COMPILE_IFELSE([AC_LANG_PROGRAM([#include <stdlib.h>
+        _Static_assert(_Generic((qsort_r),
+            void (*)(void *, size_t, size_t, void *,
+                int (*)(void *, const void *, const void *)) : 1, default: 0),
+            "Bad prototype not matched");
+    ])], [
+        AC_MSG_RESULT([broken])
+        AC_LIBOBJ([qsort_r])
+    ], [
+        AC_MSG_RESULT([no])
+        AC_DEFINE([HAVE_POSIX_QSORT_R], 1, [Defined to 1 if the qsort_r() prototype matches the upcoming POSIX standard.])
+    ])
+  ], [
+    AC_LIBOBJ([qsort_r])
+])
+
 AC_MSG_CHECKING([_Thread_local support])
 AC_COMPILE_IFELSE([AC_LANG_SOURCE([_Thread_local int foo = 0;
                    int main() {}])], [
diff --git a/include/vlc_fixups.h b/include/vlc_fixups.h
index 0263286efc..a10b8a50d9 100644
--- a/include/vlc_fixups.h
+++ b/include/vlc_fixups.h
@@ -224,11 +224,25 @@ long long atoll (const char *);
 lldiv_t lldiv (long long, long long);
 #endif
 
+
 #ifndef HAVE_QSORT_R
 void (qsort_r)(void *, size_t, size_t,
                int (*)(const void *, const void *, void *), void *);
 #endif
 
+#ifndef HAVE_POSIX_QSORT_R
+
+#ifndef _STDLIB_H_ // to prevent errors of stdlib was included before
+void (qsort_r)(void *, size_t, size_t,
+               int (*)(const void *, const void *, void *), void *);
+#endif
+
+# define qsort_r broken_qsort_r
+#include <stdlib.h>
+#undef qsort_r
+#endif
+
+
 #ifndef HAVE_STRTOF
 #ifndef __ANDROID__
 float strtof (const char *, char **);
-- 
2.17.2 (Apple Git-113)

